/*
 * $Id: HdmxWriter.java,v 1.3 2004/10/04 02:25:39 eed3si9n Exp $
 * 
 * $Copyright: copyright (c) 2003-2004, e.e d3si9n $
 * $License: 
 * This source code is part of DoubleType.
 * DoubleType is a graphical typeface designer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, e.e d3si9n gives permission to
 * link the code of this program with any Java Platform that is available
 * to public with free of charge, including but not limited to
 * Sun Microsystem's JAVA(TM) 2 RUNTIME ENVIRONMENT (J2RE),
 * and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects for all 
 * of the code used other than Java Platform. If you modify this file, 
 * you may extend this exception to your version of the file, but you are not
 * obligated to do so. If you do not wish to do so, delete this exception
 * statement from your version.
 * $
 */
 
 
package org.doubletype.ossa.truetype;

import java.io.*;
import java.util.*;

/**
 * HtmxWriter depends on GlyfWriter.
 * 
 * @author e.e
 */
public class HdmxWriter extends FontFormatWriter {
    private static TreeMap s_pointToPixel;
    
    private static void initPointToPixel() { 
        if (s_pointToPixel != null) {
            return;
        } // if
        
        s_pointToPixel = new TreeMap();
        addPointToPixel(8, 11);
        addPointToPixel(9, 12);
        addPointToPixel(10, 14);
        addPointToPixel(11, 15);
        addPointToPixel(12, 16);
        addPointToPixel(13, 18);
        addPointToPixel(14, 19);
        addPointToPixel(15, 20);
        addPointToPixel(16, 22);
        addPointToPixel(17, 23);
        addPointToPixel(18, 24);
        addPointToPixel(19, 26);
        addPointToPixel(20, 27);
        addPointToPixel(21, 28);
        addPointToPixel(22, 30);
        addPointToPixel(23, 31);
        addPointToPixel(24, 32);
        addPointToPixel(25, 34);
        addPointToPixel(26, 35);
        addPointToPixel(27, 36);
        addPointToPixel(28, 38);
        addPointToPixel(36, 48);
        addPointToPixel(48, 64);
        addPointToPixel(72, 96);
    }
    
    private static void addPointToPixel(int a_point, int a_pixel) {
	    s_pointToPixel.put(new Integer(a_point), new Integer(a_pixel));
	}
    
    public static Iterator getPointIterator() {
        initPointToPixel();
        return s_pointToPixel.keySet().iterator();
    }
    
    public static int getNumOfPointToPixel() {
        initPointToPixel();
        return s_pointToPixel.keySet().size();
    }
    
	private GlyfWriter m_glyf;
	
	public HdmxWriter(GlyfWriter a_glyf) {
		super();
		
		m_glyf = a_glyf;
		
		initPointToPixel();
	}
	
	public void write() throws IOException {		
		int numOfGlyphphs = m_glyf.numOfGlyph();
		int numOfPads = 4 - ((2 + numOfGlyphphs) % 4);
		if (numOfPads == 4) {
		    numOfPads = 0;
		} // if
		
		int size = numOfGlyphphs + 2 + numOfPads;
		
		// format version number
		writeInt16(0);
		
		// number of device records
		writeInt16(getNumOfPointToPixel());
		
		// size of device record
		writeInt32(size);
		
		Iterator i = getPointIterator();
		while (i.hasNext()) {
			Integer point = (Integer) i.next();
			// Integer pixels = getPixel(point);
			// int maxSize = pixels.intValue();
			
			// writeUInt8(point.intValue());
			// writeUInt8(maxSize);
			
			for (int j = 0; j < numOfGlyphphs; j++) {
				// TODO you have to write out precalculated advance width in pixel.
			    // writeUInt8();
			} // for j
			
			for (int j = 0; j < numOfPads; j++) {
				writeUInt8(0);
			} // for j
		} // for i
		
		pad();
	}
	
	protected String getTag() {
		return "hdmx";
	}
}
