/*
 * $Id: NameWriter.java,v 1.9 2004/06/16 07:02:52 eed3si9n Exp $
 * 
 * $Copyright: copyright (c) 2003, e.e d3si9n $
 * $License: 
 * This source code is part of DoubleType.
 * DoubleType is a graphical typeface designer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, e.e d3si9n gives permission to
 * link the code of this program with any Java Platform that is available
 * to public with free of charge, including but not limited to
 * Sun Microsystem's JAVA(TM) 2 RUNTIME ENVIRONMENT (J2RE),
 * and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects for all 
 * of the code used other than Java Platform. If you modify this file, 
 * you may extend this exception to your version of the file, but you are not
 * obligated to do so. If you do not wish to do so, delete this exception
 * statement from your version.
 * $
 */
 
 
package org.doubletype.ossa.truetype;

import java.io.*;
import java.util.*;

/**
 * @author e.e
 */
public class NameWriter extends FontFormatWriter {	
	static public final String k_regular = "Regular";
	
	String m_copyright = "\u00A9 Copyright";
	String m_familyName = "Temp";
	String m_subFamilyName = k_regular;
	String m_unique = "eed3si9n: Temp Regular: 2003";
	String m_fullFontName = "Temp";
	String m_version = "0.00";
	String m_psName = "Temp";
	String m_tradeMark = "";
	String m_manufacturer = "eed3si9n";
	String m_year = "2004";
	String m_sample = "The quick brown fox jumps over the lazy dog.";
	
	private ArrayList m_names = new ArrayList();
	private int m_platformId;
	private int m_encodingId;
	private int m_languageId;
		
	public NameWriter() {
		super();
	}
	
	private void prepare() {
		m_copyright = "\u00A9 Copyright "
			+ m_year
			+ ", "
			+ m_manufacturer
			+ ".";
		m_unique = "dtype: "
			+ m_manufacturer + ": "
			+ m_familyName + " "
			+ m_subFamilyName + ": "
			+ m_version + ": "
			+ m_year;
		m_fullFontName = m_familyName;
		m_psName = m_fullFontName;
		
		m_names.clear();
		
		m_platformId = TTName.k_microsoft;
		m_encodingId = TTName.k_winUnicodeEncode;
		m_languageId = TTName.k_winEnglishLang;
		addNames();
	}
	
	private void addNames() {
		add(m_copyright, 0);
		add(m_familyName, 1);
		add(m_subFamilyName, 2);
		add(m_unique, 3);
		add(m_fullFontName, 4);
		add(m_version, 5);
		add(m_psName, 6);
		add(m_tradeMark, 7);
		add(m_manufacturer, 8);
		// add(m_sample, 19);
	}
	
	private void add(String a_value, int a_nameId) {
		TTName name = new TTName(m_platformId,
								m_encodingId,
								m_languageId,
								a_nameId,
								a_value);
		m_names.add(name);
	}
	
	public void write() throws IOException {
		prepare();
				
		// table version number
		writeUInt16(0);
		
		int numOfNameRecord = m_names.size();
		// number of name records
		writeUInt16(numOfNameRecord);
		
		// Offset to start of string storage (from start of table).
		writeUInt16(12 * numOfNameRecord + 6);
		
		int offset = 0;
		int i;
		for (i = 0; i < numOfNameRecord; i++) {
			TTName name = (TTName) m_names.get(i);
			
			writeUInt16(name.getPlatformId());
			writeUInt16(name.getEncodingId());
			writeUInt16(name.getLanguageId());
			writeUInt16(name.getNameId());
			int len = name.getLength();
			writeUInt16(len);
			writeUInt16(offset);
			offset += len;
		} // for i
		
		for (i = 0; i < numOfNameRecord; i++) {
			TTName name = (TTName) m_names.get(i);
			m_buffer.write(name.getString());
		} // for i
				
		pad();
	}
	
	protected String getTag() {
		return "name";
	}
}
