/*
 */
package org.doubletype.ossa.test;

import junit.framework.TestCase;
import org.doubletype.ossa.*;

/**
 * @author e.e
 */
public class GlyphDisplayTest extends TestCase {
    private GlyphDisplay m_display;
    
    /**
     * Constructor for GlyphDisplayTest.
     * @param name
     */
    public GlyphDisplayTest(String name) {
        super(name);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        
        m_display = new GlyphDisplay(Engine.getSingletonInstance());
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
    
    public void testGetInitialValues() {
        assertEquals(96.0, m_display.getResolution(), 0.1);
        assertEquals(50, m_display.getZoom());
        assertEquals(9.0, m_display.getBodySize(), 0.1);
        
        // from 96 dpi and 9.0, ppem should be 12
        assertEquals(12, m_display.getPpem());
    }
    
    public void testSetResolution() {
        assertEquals(9.0, m_display.getBodySize(), 0.1);
        assertEquals(96.0, m_display.getResolution(), 0.1);
        
        // from 96 dpi and 9.0, ppem should be 12 px.
        assertEquals(12, m_display.getPpem());
        
        // if i set the resolution to 72 dpi, body size will not change
        // and ppem will change to 9 px.
        m_display.setResolution(72);
        assertEquals("wrong resolution", 72, m_display.getResolution());
        assertEquals(9.0, m_display.getBodySize(), 0.1);
        assertEquals("wrong ppem", 9, m_display.getPpem());
    }
    
    public void testSetBodySize() {
        assertEquals(9.0, m_display.getBodySize(), 0.1);
        assertEquals(96.0, m_display.getResolution(), 0.1);
        
        // from 96 dpi and 9.0, ppem should be 12 px.
        assertEquals(12, m_display.getPpem());
        
        // if i set the body size to 12pt, the resolution will not change
        // and ppem will change to 16 px.
        m_display.setBodySize(12.0);
        assertEquals("wrong body size", 12.0, m_display.getBodySize(), 0.1);
        assertEquals(96, m_display.getResolution());
        assertEquals("wrong ppem", 16, m_display.getPpem());
    }
    
    public void testSetZoom() {
        assertEquals(50, m_display.getZoom());
        m_display.setZoom(100);
        assertEquals(100, m_display.getZoom());
    }
}
