 /*
 * $Id: VarStackTest.java,v 1.7 2004/09/04 21:54:19 eed3si9n Exp $
 * 
 * $Copyright: copyright (c) 2004, e.e d3si9n $
 * $License: 
 * This source code is part of DoubleType.
 * DoubleType is a graphical typeface designer.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, e.e d3si9n gives permission to
 * link the code of this program with any Java Platform that is available
 * to public with free of charge, including but not limited to
 * Sun Microsystem's JAVA(TM) 2 RUNTIME ENVIRONMENT (J2RE),
 * and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects for all 
 * of the code used other than Java Platform. If you modify this file, 
 * you may extend this exception to your version of the file, but you are not
 * obligated to do so. If you do not wish to do so, delete this exception
 * statement from your version.
 * $
 */
 
package org.doubletype.ossa.test;

import org.doubletype.ossa.Engine;
import org.doubletype.ossa.module.*;
import org.doubletype.ossa.xml.*;
import org.doubletype.ossa.adapter.*;

import junit.framework.*;
import java.io.*;
import java.awt.geom.*;

/**
 * @author e.e
 */
public class VarStackTest extends TestCase {
	private VarStack m_target;
	private GlyphFile m_file;
	
	public VarStackTest(String arg0) {
		super(arg0);
	}
	
	public void setUp() {
		m_target = VarStack.getSingletonInstance();
		m_file = new GlyphFile(new File("glyph/test.glyph"));
		m_target.clear();
	}
	
	public void testNonExistingVar() {
		assertTrue(m_target.get("NonExisting") == 0.0);
	}
	
	public void testSize() {
		assertEquals(0, m_target.size());
	}
	
	public void testPush() {
		assertFalse(m_file.hasVariable("v1"));
		
		m_target.push(m_file);
		assertEquals(1, m_target.size());
		
		// after push params are loaded into the environment
		assertTrue(m_file.hasVariable("v1"));
		assertEquals(2.0, m_target.get("v1"), 0.01);
	}
	
	public void testSeek() {
		m_target.push(m_file);
		assertEquals(m_file, m_target.peek());
	}
	
	public void testPop() {
		m_target.push(m_file);
		assertEquals(m_file, m_target.pop());
		assertEquals(0, m_target.size());		
	}
	
	public void testSimpleGet() {
		assertEquals(0.0, m_target.get("pen"), 0.01);
		m_file.addVar("pen", 1.0);
		
		m_target.push(m_file);
		assertEquals(1.0, m_target.get("pen"), 0.01);
	}
	
	/**
	 * glyph {
	 * 	bar = 1.0;
	 * 	foo = 2.0;
	 * 	
	 *  invokeModule {
	 * 		bar = foo;
	 * 	}
	 * }
	 */
	public void testVargGet() {
		m_file.addVar("weight", 1.0);
		m_file.addVar("foo", 2.0);
		m_target.push(m_file);
		
		EModuleInvoke moduleInvoke
			= EModuleInvoke.createAt(new Point2D.Double(0, 0));
		m_file.addModule(moduleInvoke);
		XInvoke invoke = new XInvoke();
		XInvokeVarg varg = new XInvokeVarg();
		varg.setName("weight");
		varg.setSrc("foo");
		invoke.addInvokeVarg(varg);
		moduleInvoke.setInvoke(invoke);
		m_target.push(moduleInvoke);
		
		assertEquals(true, moduleInvoke.hasVariable("weight"));
		assertEquals(2.0, m_target.get("weight"), 0.01);
	}
	
	public void testModule() {
		m_target.push(m_file);
		EModuleInvoke moduleInvoke 
			= (EModuleInvoke) m_file.getGlyph().getBody().getModule(0);
		m_target.push(moduleInvoke);
		XContour contour = moduleInvoke.toContour(new AffineTransform());
		
		XContourPoint [] points = contour.getContourPoint();
		
		assertEquals(0.0, 0.0, points[0].getPoint2d());
		assertEquals(100.0, 0.0, points[1].getPoint2d());
		assertEquals(100.0, -136.0, points[2].getPoint2d());
		assertEquals(0.0, -136.0, points[3].getPoint2d());
	}
	
	public void testIncludeToContour() {
		TypefaceFile typeface = new TypefaceFile(new File("glyph/test.dtyp"));
		Engine engine = Engine.getSingletonInstance();
		engine.setTypeface(typeface);
		
		GlyphFile file2 = new GlyphFile(new File("glyph/test2.glyph"));
		
		m_target.push(file2);
		EIncludeInvoke includeInvoke
			= (EIncludeInvoke) file2.getGlyph().getBody().getInclude(0);
		m_target.push(includeInvoke);
		
		EModuleInvoke moduleInvoke 
			= (EModuleInvoke) m_file.getGlyph().getBody().getModule(0);
		m_target.push(moduleInvoke);
		
		assertEquals(0.5, m_target.get("v1"), 0.01);
		
		XContour contour = moduleInvoke.toContour(new AffineTransform());
		
		XContourPoint [] points = contour.getContourPoint();
		
		assertEquals(0.0, 0.0, points[0].getPoint2d());
		assertEquals(100.0, 0.0, points[1].getPoint2d());
		assertEquals(100.0, -34.0, points[2].getPoint2d());
		assertEquals(0.0, -34.0, points[3].getPoint2d());
	}
	
	/**
	 * test XPoint2d object
	 * @param a_x expected x value
	 * @param a_y expected y value
	 * @param a_point actual point
	 */
	public void assertEquals(double a_x, double a_y, XPoint2d a_point) {
		assertEquals(a_x, a_point.getX(), 0.01);
		assertEquals(a_y, a_point.getY(), 0.01);
	}
}
